@@ -1,25 +1,28 @@
 /*
 ################################################################################
-# 
-# Copyright(c) Realtek Semiconductor Corp. All rights reserved.
-# 
-# This program is free software; you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License as published by the Free 
-# Software Foundation; either version 2 of the License, or (at your option) 
+#
+# r8101 is the Linux device driver released for RealTek RTL8101E, RTL8102E,
+# and RTL8103E Fast Ethernet controllers with PCI-Express interface.
+#
+# Copyright(c) 2010 Realtek Semiconductor Corp. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
-# 
-# This program is distributed in the hope that it will be useful, but WITHOUT 
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 # more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc., 59 
-# Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# 
-# The full GNU General Public License is included in this distribution in the
-# file called LICENSE.
-# 
+# this program; if not, see <http://www.gnu.org/licenses/>.
+#
+# Author:
+# Realtek NIC software team <nicfae@realtek.com>
+# No. 2, Innovation Road II, Hsinchu Science Park, Hsinchu 300, Taiwan
+#
 ################################################################################
 */
 
@@ -32,10 +35,11 @@
 #define CHECKSUM_PARTIAL CHECKSUM_HW
 #endif
 
-#ifndef IRQ_HANDLED
-#define irqreturn_t void
-#define IRQ_HANDLED
-#define IRQ_NONE
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+ #define irqreturn_t void
+ #define IRQ_HANDLED	1
+ #define IRQ_NONE	0
+ #define IRQ_RETVAL(x)
 #endif
 
 #ifndef HAVE_FREE_NETDEV
@@ -54,23 +58,60 @@
 #define SA_SHIRQ IRQF_SHARED
 #endif
 
-//Due to the hardware design of RTL8101E, the low 32 bit address of receive
-//buffer must be 8-byte alignment.
-#undef NET_IP_ALIGN
-#define NET_IP_ALIGN 8
-
-#define NODE_ADDRESS_SIZE 6
-
-#ifdef CONFIG_R8101_NAPI
-#define NAPI_SUFFIX	"-NAPI"
-#else
-#define NAPI_SUFFIX	""
+#ifndef NETIF_F_GSO
+#define gso_size	tso_size
+#define gso_segs	tso_segs
 #endif
 
-#define RTL8101_VERSION "1.011.00" NAPI_SUFFIX
+#ifdef HAVE_NET_DEVICE_OPS
+	#define RTL_NET_DEVICE_OPS(ops)	dev->netdev_ops=&ops
+#else
+	#ifdef CONFIG_NET_POLL_CONTROLLER
+		#define RTL_NET_POLL_CONTROLLER dev->poll_controller=rtl8101_netpoll
+	#else
+		#define RTL_NET_POLL_CONTROLLER
+	#endif
+
+	#ifdef CONFIG_R8101_VLAN
+		#define RTL_SET_VLAN dev->vlan_rx_register=rtl8101_vlan_rx_register
+	#else
+		#define RTL_SET_VLAN
+	#endif
+
+	#define RTL_NET_DEVICE_OPS(ops)	dev->open=rtl8101_open; \
+					dev->hard_start_xmit=rtl8101_start_xmit; \
+					dev->get_stats=rtl8101_get_stats; \
+					dev->stop=rtl8101_close; \
+					dev->tx_timeout=rtl8101_tx_timeout; \
+					dev->set_multicast_list=rtl8101_set_rx_mode; \
+					dev->change_mtu=rtl8101_change_mtu; \
+					dev->set_mac_address=rtl8101_set_mac_address; \
+					dev->do_ioctl=rtl8101_do_ioctl; \
+					RTL_NET_POLL_CONTROLLER; \
+					RTL_SET_VLAN;
+#endif //HAVE_NET_DEVICE_OPS
+
+//Due to the hardware design of RTL8101E, the low 32 bit address of receive
+//buffer must be 8-byte alignment.
+#define RTK_RX_ALIGN		8
+
+#define NODE_ADDRESS_SIZE	6
+
+#ifdef CONFIG_R8101_NAPI
+#define NAPI_SUFFIX		"-NAPI"
+#else
+#define NAPI_SUFFIX		""
+#endif
+
+#define RTL8101_VERSION "1.018.00" NAPI_SUFFIX
 #define MODULENAME "r8101"
 #define PFX MODULENAME ": "
 
+#define GPL_CLAIM "\
+r8101  Copyright (C) 2010  Realtek NIC software team <nicfae@realtek.com> \n \
+This program comes with ABSOLUTELY NO WARRANTY; for details, please see <http://www.gnu.org/licenses/>. \n \
+This is free software, and you are welcome to redistribute it under certain conditions; see <http://www.gnu.org/licenses/>. \n"
+
 #ifdef RTL8101_DEBUG
 #define assert(expr) \
         if(!(expr)) {					\
@@ -122,7 +163,7 @@
 #define NUM_TX_DESC	1024	/* Number of Tx descriptor registers */
 #define NUM_RX_DESC	1024	/* Number of Rx descriptor registers */
 
-#define RX_BUF_SIZE	1519	/* Rx Buffer size */
+#define RX_BUF_SIZE	0x05EF	/* Rx Buffer size */
 #define R8101_TX_RING_BYTES	(NUM_TX_DESC * sizeof(struct TxDesc))
 #define R8101_RX_RING_BYTES	(NUM_RX_DESC * sizeof(struct RxDesc))
 
@@ -132,6 +173,8 @@
 #define RTL8101_ESD_TIMEOUT		(2*HZ)
 #define RTL8101_LINK_DOWN_CHECK_TIMEOUT	(10*HZ)
 
+#define RTL_PCI_VENDOR_ID	0x10ec
+
 /* write/read MMIO register */
 #define RTL_W8(reg, val8)	writeb ((val8), ioaddr + (reg))
 #define RTL_W16(reg, val16)	writew ((val16), ioaddr + (reg))
@@ -183,6 +226,7 @@
 	#define	RTLDEV	dev
 #endif	//LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)
 /*****************************************************************************/
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
 	typedef struct net_device *napi_ptr;
 	typedef int *napi_budget;
@@ -192,7 +236,7 @@
 								ndev->weight=weig;
 	#define RTL_NAPI_QUOTA(budget, ndev)			min(*budget, ndev->quota)
 	#define RTL_GET_PRIV(stuct_ptr, priv_struct)		netdev_priv(stuct_ptr)
-	#define RTL_GET_NETDEV(priv_ptr)			
+	#define RTL_GET_NETDEV(priv_ptr)
 	#define RTL_RX_QUOTA(ndev, budget)			ndev->quota
 	#define RTL_NAPI_QUOTA_UPDATE(ndev, work_done, budget)	*budget -= work_done;	\
 								ndev->quota -= work_done;
@@ -212,13 +256,26 @@
 	#define RTL_GET_NETDEV(priv_ptr)			struct net_device *dev = priv_ptr->dev;
 	#define RTL_RX_QUOTA(ndev, budget)			budget
 	#define RTL_NAPI_QUOTA_UPDATE(ndev, work_done, budget)
+ #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
 	#define RTL_NETIF_RX_COMPLETE(dev, napi)		netif_rx_complete(dev, napi)
 	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		netif_rx_schedule_prep(dev, napi)
 	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__netif_rx_schedule(dev, napi)
+ #endif
+ #if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,29)
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		netif_rx_complete(napi)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		netif_rx_schedule_prep(napi)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__netif_rx_schedule(napi)
+ #endif
+ #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		napi_complete(napi)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		napi_schedule_prep(napi)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__napi_schedule(napi)
+ #endif
 	#define RTL_NAPI_RETURN_VALUE work_done
 	#define RTL_NAPI_ENABLE(dev, napi)			napi_enable(napi)
 	#define RTL_NAPI_DISABLE(dev, napi)			napi_disable(napi)
 #endif	//LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
+
 /*****************************************************************************/
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
 #ifdef __CHECKER__
@@ -344,12 +401,6 @@ static inline int _kc_pci_dma_mapping_error(dma_addr_t dma_addr)
 
 /* ----------------------------------------------------------- */
 
-
-enum RTL8101_VLAN_TYPE {
-	RTL_S_VLAN = 1,
-	RTL_H_VLAN = 2,
-};
-
 enum RTL8101_DSM_STATE {
 	DSM_MAC_INIT = 1,
 	DSM_NIC_GOTO_D3 = 2,
@@ -390,6 +441,8 @@ enum RTL8101_registers {
 	MACDBG = 0x6D,
 	GPIO = 0x6E,
 	PMCH = 0x6F,
+	ERIDR = 0x70,
+	ERIAR = 0x74,
 	EPHYAR = 0x80,
 	DBG_reg = 0xD1,
 	RxMaxSize = 0xDA,
@@ -397,11 +450,8 @@ enum RTL8101_registers {
 	IntrMitigate = 0xE2,
 	RxDescAddrLow = 0xE4,
 	RxDescAddrHigh = 0xE8,
-	Reserved1 = 0xEC,
-	FuncEvent = 0xF0,
-	FuncEventMask = 0xF4,
-	FuncPresetState = 0xF8,
-	FuncForceEvent = 0xFC,
+	MTPS = 0xEC,
+	PHYIO = 0xF8,
 };
 
 enum RTL8101_register_content {
@@ -530,6 +580,14 @@ enum RTL8101_register_content {
 	PHYAR_Reg_shift = 16,
 	PHYAR_Data_Mask = 0xffff,
 
+	/* PHY IO access */
+	PHYIO_Flag = 0x80000000,
+	PHYIO_Write = 0x80000000,
+	PHYIO_Read = 0x00000000,
+	PHYIO_Reg_Mask = 0x1f,
+	PHYIO_Reg_shift = 16,
+	PHYIO_Data_Mask = 0xffff,
+
 	/* EPHY access */
 	EPHYAR_Flag = 0x80000000,
 	EPHYAR_Write = 0x80000000,
@@ -538,7 +596,7 @@ enum RTL8101_register_content {
 	EPHYAR_Reg_shift = 16,
 	EPHYAR_Data_Mask = 0xffff,
 
-	/* CSI access */	
+	/* CSI access */
 	CSIAR_Flag = 0x80000000,
 	CSIAR_Write = 0x80000000,
 	CSIAR_Read = 0x00000000,
@@ -546,8 +604,20 @@ enum RTL8101_register_content {
 	CSIAR_ByteEn_shift = 12,
 	CSIAR_Addr_Mask = 0x0fff,
 
+	/* ERI access */
+	ERIAR_Flag = 0x80000000,
+	ERIAR_Write = 0x80000000,
+	ERIAR_Read = 0x00000000,
+	ERIAR_Addr_Align = 4, /* ERI access register address must be 4 byte alignment */
+	ERIAR_ExGMAC = 0,
+	ERIAR_MSIX = 1,
+	ERIAR_ASF = 2,
+	ERIAR_Type_shift = 16,
+	ERIAR_ByteEn = 0x0f,
+	ERIAR_ByteEn_shift = 12,
+
 	/* GPIO */
-	GPIO_en = (1 << 0),	
+	GPIO_en = (1 << 0),
 
 };
 
@@ -599,6 +669,62 @@ enum _DescStatusBit {
 	/*@@@@@@ offset 4 of rx descriptor => bits for RTL8102E only		end @@@@@@*/
 };
 
+struct rtl8101_counters {
+	u64	tx_packets;
+	u64	rx_packets;
+	u64	tx_errors;
+	u32	rx_errors;
+	u16	rx_missed;
+	u16	align_errors;
+	u32	tx_one_collision;
+	u32	tx_multi_collision;
+	u64	rx_unicast;
+	u64	rx_broadcast;
+	u32	rx_multicast;
+	u16	tx_aborted;
+	u16	tx_underun;
+};
+
+enum wol_capability {
+	WOL_DISABLED = 0,
+	WOL_ENABLED = 1
+};
+
+enum bits {
+	BIT_0 = (1 << 0),
+	BIT_1 = (1 << 1),
+	BIT_2 = (1 << 2),
+	BIT_3 = (1 << 3),
+	BIT_4 = (1 << 4),
+	BIT_5 = (1 << 5),
+	BIT_6 = (1 << 6),
+	BIT_7 = (1 << 7),
+	BIT_8 = (1 << 8),
+	BIT_9 = (1 << 9),
+	BIT_10 = (1 << 10),
+	BIT_11 = (1 << 11),
+	BIT_12 = (1 << 12),
+	BIT_13 = (1 << 13),
+	BIT_14 = (1 << 14),
+	BIT_15 = (1 << 15),
+	BIT_16 = (1 << 16),
+	BIT_17 = (1 << 17),
+	BIT_18 = (1 << 18),
+	BIT_19 = (1 << 19),
+	BIT_20 = (1 << 20),
+	BIT_21 = (1 << 21),
+	BIT_22 = (1 << 22),
+	BIT_23 = (1 << 23),
+	BIT_24 = (1 << 24),
+	BIT_25 = (1 << 25),
+	BIT_26 = (1 << 26),
+	BIT_27 = (1 << 27),
+	BIT_28 = (1 << 28),
+	BIT_29 = (1 << 29),
+	BIT_30 = (1 << 30),
+	BIT_31 = (1 << 31)
+};
+
 #define RsvdMask	0x3fffc000
 
 struct TxDesc {
@@ -664,7 +790,6 @@ struct rtl8101_private {
 	unsigned rx_buf_sz;
 	struct timer_list aspm_timer;
 	struct timer_list esd_timer;
-	struct timer_list link_down_check_timer;
 	struct pci_resource pci_cfg_space;
 	unsigned int esd_flag;
 	unsigned int pci_cfg_is_read;
@@ -672,11 +797,12 @@ struct rtl8101_private {
 	u16 intr_mask;
 	int phy_auto_nego_reg;
 	uint8_t mac_addr[NODE_ADDRESS_SIZE];
+	struct rtl8101_counters *counters;
+	dma_addr_t paddr;
 
 #ifdef CONFIG_R8101_VLAN
 	struct vlan_group *vlgrp;
 #endif
-	int vlan_type;
 	int (*set_speed)(struct net_device *, u8 autoneg, u16 speed, u8 duplex);
 	void (*get_settings)(struct net_device *, struct ethtool_cmd *);
 	void (*phy_reset_enable)(struct net_device *);
